@@ -7,3 +7,7 @@ class NoSchemaError(Exception):
 
 class BadTomlError(Exception):
     """Raised when loading from an invalid toml source is attempted."""
+
+
+class UnsupportedConfigError(Exception):
+    """Raised when a config is attempted to be parsed but no parser is registered for it."""
